import jwt
from fastapi import Depends, HTTPException
from fastapi.security import OAuth2AuthorizationCodeBearer
from jwt import PyJWKClient
from pydantic import BaseModel

# Your tenant ID and the application (client) ID
TENANT_ID = "e420b389-a245-4a34-aa22-cf3277c4b1d4"
CLIENT_ID = "41b2bef7-0e67-4390-86bb-97b2843d121e"

oauth2_scheme = OAuth2AuthorizationCodeBearer(
    authorizationUrl=f"https://login.microsoftonline.com/{TENANT_ID}/oauth2/v2.0/authorize",
    tokenUrl=f"https://login.microsoftonline.com/{TENANT_ID}/oauth2/v2.0/token",
)

# Public keys endpoint for your Azure AD tenant
jwks_url = "https://metricplatform.b2clogin.com/metricplatform.onmicrosoft.com/b2c_1a_displaycontrol_sendgrid_signin/discovery/v2.0/keys"


class User(BaseModel):
    oid: str
    email: str
    city: str


def get_current_active_user(token: str = Depends(oauth2_scheme)):
    """
    Get the current active user from the token.
    Also validates the token.
    """
    try:
        # Fetch the JWKS keys
        jwks_client = PyJWKClient(jwks_url)
        signing_key = jwks_client.get_signing_key_from_jwt(token)

        # decoded_token = msal.oauth2cli.oidc.decode_id_token(token)

        # Decode the token
        payload = jwt.decode(
            token, signing_key.key, algorithms=["RS256"], audience=CLIENT_ID
        )

        user = User(
            oid=payload["oid"],
            email=payload.get("email", ""),
            city=payload.get("city", ""),
        )
        return user
    except jwt.ExpiredSignatureError:
        raise HTTPException(status_code=401, detail="Token has expired")
    except jwt.InvalidTokenError as e:
        raise HTTPException(status_code=401, detail=f"Invalid token: {str(e)}")

    except Exception as e:
        raise HTTPException(status_code=401, detail=f"Invalid token: {str(e)}")
